"""Module for Certificate Authority abstract base class
"""
__author__ = "P J Kershaw"
__date__ = "20/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: base.py 8238 2012-10-29 13:33:50Z pjkersha $"
from ConfigParser import ConfigParser, SafeConfigParser
from os import path
from abc import ABCMeta, abstractmethod
import logging
log = logging.getLogger(__name__)

from OpenSSL import crypto


class CertificateAuthorityError(Exception):
    """Base class for exceptions for CertificateAuthority class"""
        
    
class CertificateAuthorityConfigError(CertificateAuthorityError):
    """Error reading options from config file"""
    
    
class CertificateIssuingError(CertificateAuthorityError):
    """Error issuing a certificate"""
    
    
class AbstractCertificateAuthority(object):
    """Provide basic functionality for a Certificate Authority"""
    CERTIFICATE_VERSION2 = 1
    CERTIFICATE_VERSION3 = 2
    CERT_FILEPATH_OPTNAME = "cert_filepath"
    PRIKEY_FILEPATH_OPTNAME = "key_filepath"
    PRIKEY_PASSWD_OPTNAME = "key_passwd"
    SERIAL_NUM_DEFAULT = 0L
    MIN_KEY_NBITS_DEFAULT = 2048
    
    __metaclass__ = ABCMeta
    __slots__ = (
        '__cert', 
        '__key', 
        '__serial_num_counter', 
        '__min_key_nbits',
        '__serial_filepath'
    )
    
    def __init__(self):
        """Create certificate authority instance"""
        self.__cert = None
        self.__key = None
        self.__serial_num_counter = self.__class__.SERIAL_NUM_DEFAULT
        self.__min_key_nbits = self.__class__.MIN_KEY_NBITS_DEFAULT
        self.__serial_filepath = None
    
    @abstractmethod
    def issue_certificate(self, cert_req):
        """
        Generate a certificate given a certificate request.
    
        @param cert_req: Certificate request to use
        """
        
    @classmethod
    def from_config(cls, cfg, **kw):
        '''Alternative constructor makes object from config file settings
        @type cfg: basestring / ConfigParser derived type
        @param cfg: configuration file path or ConfigParser type object
        @rtype: ndg.saml.saml2.binding.soap.client.SOAPBinding or derived type
        @return: new instance of this class
        '''
        obj = cls()
        obj.parse_config(cfg, **kw)
        
        return obj

    def parse_config(self, cfg, prefix='', section='DEFAULT'):
        '''Read config file settings
        @type cfg: basestring /ConfigParser derived type
        @param cfg: configuration file path or ConfigParser type object
        @type prefix: basestring
        @param prefix: prefix for option names e.g. "attributeQuery."
        @type section: baestring
        @param section: configuration file section from which to extract
        parameters.
        '''  
        if isinstance(cfg, basestring):
            config_file_path = path.expandvars(cfg)
            here_dir = path.dirname(config_file_path)
            _cfg = SafeConfigParser(defaults={'here':here_dir})
            _cfg.optionxform = str

            _cfg.read(config_file_path)
            
        elif isinstance(cfg, ConfigParser):
            _cfg = cfg   
        else:
            raise AttributeError('Expecting basestring or ConfigParser type '
                                 'for "cfg" attribute; got %r type' % type(cfg))
                        
        # Get items for this section as a dictionary so that parse_keywords can
        # used to update the object
        kw = dict([(opt_name, val) for opt_name, val in _cfg.items(section)
                   if opt_name != 'here'])
        
        # ... but first get file path settings which aren't included as 
        # instance variables
        cert_filepath_opt = prefix + self.__class__.CERT_FILEPATH_OPTNAME
        prikey_filepath_opt = prefix + self.__class__.PRIKEY_FILEPATH_OPTNAME
        prikey_file_passwd_opt = prefix + self.__class__.PRIKEY_PASSWD_OPTNAME
        
        try:
            cert_filepath = kw.pop(cert_filepath_opt)
            prikey_filepath = kw.pop(prikey_filepath_opt)
            
        except KeyError, e:
            raise CertificateAuthorityConfigError('Missing option from config '
                                                  '%s' % str(e))

        # Password does not need to be set and can default to None
        prikey_file_passwd = kw.pop(prikey_file_passwd_opt, None)
            
        # Set 'cert' and 'key' attributes from equivalent files
        self.parse_files(cert_filepath, prikey_filepath, 
                         key_file_passwd=prikey_file_passwd)
        
        # Take prefix setting from config if set otherwise default to input
        # setting made to this method
        if 'prefix' not in kw and prefix:
            kw['prefix'] = prefix
        
        # Prune cert and key settings as these cannot be intelligently read from
        # a config file.
        for key in ('cert', 'key'):
            if key in kw:
                del key
                    
        self.parse_keywords(**kw)
        
        if self.serial_filepath is not None:
            self.init_serial_file()
        
    def parse_keywords(self, prefix='', **kw):
        """Update object from input keywords
        
        @type prefix: basestring
        @param prefix: if a prefix is given, only update self from kw items 
        where keyword starts with this prefix
        @type kw: dict
        @param kw: items corresponding to class instance variables to 
        update.  Keyword names must match their equivalent class instance 
        variable names.  However, they may prefixed with <prefix>
        """        
        prefix_len = len(prefix)
        for opt_name, val in kw.items():
            if prefix:
                # Filter attributes based on prefix
                if opt_name.startswith(prefix):
                    setattr(self, opt_name[prefix_len:], val)
            else:
                # No prefix set - attempt to set all attributes   
                setattr(self, opt_name, val)
                
    @classmethod
    def from_keywords(cls, prefix='', **kw):
        """Create a new instance initialising instance variables from the 
        keyword inputs
        @type prefix: basestring
        @param prefix: if a prefix is given, only update self from kw items 
        where keyword starts with this prefix
        @type kw: dict
        @param kw: items corresponding to class instance variables to 
        update.  Keyword names must match their equivalent class instance 
        variable names.  However, they may prefixed with <prefix>
        @return: new instance of this class
        @rtype: ca.base.AbstractCertificateAuthority derived type
        """
        obj = cls()
        
        # ... but first get file path settings which aren't included as 
        # instance variables
        cert_filepath_opt = prefix + cls.CERT_FILEPATH_OPTNAME
        prikey_filepath_opt = prefix + cls.PRIKEY_FILEPATH_OPTNAME
        prikey_file_passwd_opt = prefix + cls.PRIKEY_PASSWD_OPTNAME
        
        cert_filepath = kw.pop(cert_filepath_opt, None)
        if cert_filepath is None and 'cert' not in kw:
            raise CertificateAuthorityConfigError(
                    "No 'cert' or %r option set" % cls.CERT_FILEPATH_OPTNAME)
   
        prikey_filepath = kw.pop(prikey_filepath_opt, None)
        if prikey_filepath is None and 'key' not in kw:
            raise CertificateAuthorityConfigError(
                    "No 'key' or %r option set" % cls.PRIKEY_FILEPATH_OPTNAME)
            
        # Password does not need to be set and can default to None
        prikey_file_passwd = kw.pop(prikey_file_passwd_opt, None)
            
        # Set 'cert' and 'key' attributes from equivalent files
        if cert_filepath and prikey_filepath:
            obj.parse_files(cert_filepath, prikey_filepath, 
                            key_file_passwd=prikey_file_passwd)

        obj.parse_keywords(prefix=prefix, **kw)
        
        return obj
        
    def parse_files(self, cert_filepath, key_filepath, key_file_passwd=None):
        """Read certificate and private key files setting instance variables
        """
        args = crypto.FILETYPE_PEM, open(key_filepath).read()
        if key_file_passwd:
            args += (key_file_passwd, )
            
        self.key = crypto.load_privatekey(*args)
        
        self.cert = crypto.load_certificate(crypto.FILETYPE_PEM, 
                                            open(cert_filepath).read())

        
    @classmethod
    def from_files(cls, cert_filepath, key_filepath, key_file_passwd=None):
        """Construct new instance certificate and private key files
        @return: new certificate authority instance
        """
        ca = cls()
        ca.parse_files(cert_filepath, key_filepath, 
                       key_file_passwd=key_file_passwd)
        
        return ca
    
    @property
    def cert(self):
        return self.__cert

    @cert.setter
    def cert(self, value):
        if not isinstance(value, crypto.X509):
            raise TypeError('Expecting OpenSSL.crypto.X509 type for "cert" '
                            'got %r type' % type(value))

        self.__cert = value
        
    @property
    def key(self):
        """CA private key"""
        return self.__key

    @key.setter
    def key(self, value):
        if not isinstance(value, crypto.PKey):
            raise TypeError('Expecting OpenSSL.crypto.PKey type for "key" '
                            'got %r type' % type(value))
        self.__key = value
        
    @property
    def serial_num_counter(self):
        """Certificate serial number"""
        return self.__serial_num_counter

    @serial_num_counter.setter
    def serial_num_counter(self, value):
        if not isinstance(value, (long, int)):
            raise TypeError('Expecting int or long type for '
                            '"serial_num_counter" got %r type' % type(value))
        self.__serial_num_counter = long(value)
        
    @property
    def serial_filepath(self):
        return self.__serial_filepath
    
    @serial_filepath.setter
    def serial_filepath(self, value):
        if not isinstance(value, basestring):
            raise TypeError('Expecting string type for "serial_filepath" '
                            'got %r type' % type(value))
        self.__serial_filepath = value
        
    @property
    def min_key_nbits(self):
        """Minimum number of bits required for key in certificate request"""
        return self.__min_key_nbits

    @min_key_nbits.setter
    def min_key_nbits(self, value):
        if not isinstance(value, (long, int, basestring)):
            raise TypeError('Expecting int or long type for "min_key_nbits" '
                            'got %r type' % type(value))
        self.__min_key_nbits = long(value)
          
    def _read_serial_file(self):
        '''Read serial number from serial file'''
        serial_file = open(self.serial_filepath, 'r')
        
        try:
            self.serial_num_counter = long(serial_file.read())
        finally:
            serial_file.close()
                    
    def _write_serial_file(self):
        '''Write serial number to serial file'''
        serial_file = open(self.serial_filepath, 'w')
        
        try:
            serial_file.write("%2l" % self.serial_num_counter)
        finally:
            serial_file.close()
            
    def init_serial_file(self):
        '''Initialise serial file if doesn't already exist'''
        if not path.exists(self.serial_filepath):
            self._write_serial_file()



"""Certificate Authority implementation using openssl excutable callout
"""
__author__ = "P J Kershaw"
__date__ = "19/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: callout_impl.py 8238 2012-10-29 13:33:50Z pjkersha $"
import tempfile
import string
import subprocess
import os
import logging
log = logging.getLogger(__name__)
    
from OpenSSL import crypto

from contrail.security.ca.base import (AbstractCertificateAuthority,
                                       CertificateIssuingError)


class CertificateAuthorityWithCallout(AbstractCertificateAuthority):
    '''Certificate Authority implementation using a callout to an OpenSSL
    executable to issue certificates.  This enables more flexibility in how
    certificates are configured
    '''
    __slots__ = AbstractCertificateAuthority.__slots__ + ('__cert_issue_cmd', )
    
    def __init__(self):
        super(CertificateAuthorityWithCallout, self).__init__()
        self.__cert_issue_cmd = None
        
    def issue_certificate(self, cert_req):
        '''Certificate issuing from a callout to an OpenSSL executable
    
        @param cert_req: Certificate request to use
        @return: output certificate
        ''' 
        try:       
            out_cert_file = tempfile.NamedTemporaryFile(delete=False)
            in_csr_file = tempfile.NamedTemporaryFile(delete=False)
            s_cert_req = crypto.dump_certificate_request(crypto.FILETYPE_PEM, 
                                                         cert_req)
            in_csr_file.seek(0)
            in_csr_file.write(s_cert_req)
            in_csr_file.close()
            
            
            cmd_tmpl = string.Template(self.cert_issue_cmd)
            populated_cmd = cmd_tmpl.substitute(in_csr=in_csr_file.name,
                                                out_cert=out_cert_file.name)
            
            log.debug('Executing command to issue certificate: %r', 
                      populated_cmd)
            
            cmd_args = populated_cmd.split()
                    
            proc = subprocess.Popen(cmd_args, 
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
    
            stdoutdata, stderrdata = proc.communicate()           
            if proc.returncode == 0:
                if stdoutdata:
                    log.info('stdout message: \'%s\'; for command: %r', 
                             stdoutdata,
                             populated_cmd)
            
                if stderrdata:
                    log.info('stderr message: \'%s\'; for command: %r', 
                              stderrdata,
                              populated_cmd)
 
                out_cert = crypto.load_certificate(
                                               crypto.FILETYPE_PEM, 
                                               open(out_cert_file.name).read())
            else:
                error_msg = ''
                if stdoutdata:
                    error_msg += ('stdout message: \'%s\'; for command: %r' %
                                                (stdoutdata, populated_cmd))
            
                if stderrdata:
                    error_msg += ('stderr message: \'%s\'; for command: %r' % 
                                                (stderrdata, populated_cmd))
                    
                raise CertificateIssuingError('Error issuing certificate: %s' %
                                              error_msg)
        finally:
            os.unlink(in_csr_file.name)
            os.unlink(out_cert_file.name)
            
        return out_cert
    
    @property
    def cert_issue_cmd(self):
        return self.__cert_issue_cmd
    
    @cert_issue_cmd.setter
    def cert_issue_cmd(self, value):
        if not isinstance(value, basestring):
            raise TypeError('Expecting string type for "cert_issue_cmd" '
                            'got %r type' % type(value))
        self.__cert_issue_cmd = value


"""Module for Certificate Authority OpenSSL callout implementation unit tests

This requires the openssl executable installed and on the system path of the 
host
"""
__author__ = "P J Kershaw"
__date__ = "19/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: test_ca_callout_impl.py 8241 2012-10-29 14:53:16Z pjkersha $"
import logging
logging.basicConfig(level=logging.DEBUG)
log = logging.getLogger(__name__)
import unittest
import uuid
from os import path, environ

from OpenSSL import crypto

from contrail.security.ca.callout_impl import CertificateAuthorityWithCallout
from contrail.security.ca.test import CertificateAuthorityBaseTestCase, THIS_DIR

# Configure environment for openssl config file - see ./ca_config/ca_config.cfg
environ['CONTRAIL_CA_TEST_DIR'] = THIS_DIR


class CertificateAuthorityWithCalloutTestCase(CertificateAuthorityBaseTestCase):
    """Test certificate authority class"""
    CERT_AUTHORITY_CLASS = CertificateAuthorityWithCallout
    CFG_FILEPATH = path.join(THIS_DIR, 'callout_ca.cfg')

    def test01_create_from_keywords(self):
        test_ca_cfg_filepath = path.join(THIS_DIR, 'ca_config', 'ca_config.cfg')
        cert_issue_cmd = (
            'openssl ca -key file.key -cert file.crt -config '
            '%s -days 365 -in $in_csr -out $out_cert '
            '-batch'
        ) % test_ca_cfg_filepath

        ca = CertificateAuthorityWithCallout.from_keywords(
                            min_key_nbits=4096,
                            cert_filepath=self.__class__.CA_CERT_FILEPATH, 
                            key_filepath=self.__class__.CA_KEY_FILEPATH, 
                            key_passwd=self.__class__.CA_KEY_FILE_PASSWD,
                            cert_issue_cmd=cert_issue_cmd)
        self.assert_(ca, 'null ca object')
        
        self.assertEqual(ca.min_key_nbits, 4096, 
                         'Error setting serial_num_counter')
        
        self.assertRaises(TypeError, 
                          CertificateAuthorityWithCallout.from_keywords,
                          cert_filepath=self.__class__.CA_CERT_FILEPATH, 
                          key_filepath=self.__class__.CA_KEY_FILEPATH, 
                          key_passwd=self.__class__.CA_KEY_FILE_PASSWD,
                          cert_issue_cmd=cert_issue_cmd,
                          min_key_nbits=None)
        
    def test02_create_from_files(self):
        ca = CertificateAuthorityWithCallout.from_files(
                            self.__class__.CA_CERT_FILEPATH, 
                            self.__class__.CA_KEY_FILEPATH, 
                            key_file_passwd=self.__class__.CA_KEY_FILE_PASSWD)
        self.assert_(ca, 'null ca object')
        self.assertIsInstance(ca.cert, crypto.X509, 
                              'ca.cert is not an X509 instance')
        self.assertIsInstance(ca.key, crypto.PKey, 
                              'ca.key is not an PKey instance')
        
    def test03_create_from_config(self):
        ca = CertificateAuthorityWithCallout.from_config(
                                                self.__class__.CFG_FILEPATH)
        self.assert_(ca, 'null ca object')
        self.assertIsInstance(ca.cert, crypto.X509, 
                              'ca.cert is not an X509 instance')
        self.assertIsInstance(ca.key, crypto.PKey, 
                              'ca.key is not an PKey instance')
        self.assert_(ca.cert_issue_cmd, 'Missing openssl issuing command')
    
    def test04_issue_certificate(self):
        # Use random dn to avoid error overwriting existing entry in the db
        dn = {'CN': str(uuid.uuid4()), 'O': 'NDG', 'OU': 'Security'}
        cert_req = self.__class__.create_cert_req(dn)[-1]
        ca = CertificateAuthorityWithCallout.from_config(
                                                self.__class__.CFG_FILEPATH)
        cert = ca.issue_certificate(cert_req)
        self.assert_(cert, 'Null output certificate')
        log.debug('Issued new certificate: %r', cert.get_subject())


if __name__ == "__main__":
    unittest.main()


"""ca package contains Certificate Authority classes and test package
"""
__author__ = "P J Kershaw"
__date__ = "10/08/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: $"



"""Certificate Authority utilities to support X.509 SubjectAltNames using
pyasn1

Contrail Project
"""
__author__ = "P J Kershaw"
__date__ = "19/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"
from pyasn1.type import univ, constraint, char, namedtype
from pyasn1.type import tag

MAX = 64


class DirectoryString(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            'teletexString', char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType(
            'printableString', char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType(
            'universalString', char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType(
            'utf8String', char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType(
            'bmpString', char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType(
            'ia5String', char.IA5String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        )


class AttributeValue(DirectoryString):
    pass


class AttributeType(univ.ObjectIdentifier):
    pass


class AttributeTypeAndValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('type', AttributeType()),
        namedtype.NamedType('value', AttributeValue()),
        )


class RelativeDistinguishedName(univ.SetOf):
    componentType = AttributeTypeAndValue()

class RDNSequence(univ.SequenceOf):
    componentType = RelativeDistinguishedName()


class Name(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('', RDNSequence()),
        )


class Extension(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('extnID', univ.ObjectIdentifier()),
        namedtype.DefaultedNamedType('critical', univ.Boolean('False')),
        namedtype.NamedType('extnValue', univ.OctetString()),
        )


class Extensions(univ.SequenceOf):
    componentType = Extension()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class GeneralName(univ.Choice):
    componentType = namedtype.NamedTypes(
        # namedtype.NamedType('otherName', AnotherName()),
        namedtype.NamedType('rfc822Name', char.IA5String().subtype(
                            implicitTag=tag.Tag(tag.tagClassContext,
                                                tag.tagFormatSimple, 1))),
        namedtype.NamedType('dNSName', char.IA5String().subtype(
                            implicitTag=tag.Tag(tag.tagClassContext,
                                                tag.tagFormatSimple, 2))),
        # namedtype.NamedType('x400Address', ORAddress()),
        namedtype.NamedType('directoryName', Name().subtype(
                            implicitTag=tag.Tag(tag.tagClassContext,
                                                tag.tagFormatSimple, 4))),
        # namedtype.NamedType('ediPartyName', EDIPartyName()),
        namedtype.NamedType('uniformResourceIdentifier', char.IA5String().subtype(
                            implicitTag=tag.Tag(tag.tagClassContext,
                                                tag.tagFormatSimple, 6))),
        namedtype.NamedType('iPAddress', univ.OctetString().subtype(
                            implicitTag=tag.Tag(tag.tagClassContext,
                                                tag.tagFormatSimple, 7))),
        namedtype.NamedType('registeredID', univ.ObjectIdentifier().subtype(
                            implicitTag=tag.Tag(tag.tagClassContext,
                                                tag.tagFormatSimple, 8))),
        )


class GeneralNames(univ.SequenceOf):
    componentType = GeneralName()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class SubjectAltName(GeneralNames):
    pass




"""Module for Certificate Authority default implementation unit tests
"""
__author__ = "P J Kershaw"
__date__ = "19/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: $"
import warnings
import logging
logging.basicConfig(level=logging.DEBUG)

from os import path
import unittest

from OpenSSL import crypto

PYASN1_SUPPORT = True
try:
    from pyasn1.codec.der.decoder import decode
    
except ImportError:
    warnings.warn('Skipping certificate extensions test, pyasn1 is not '
                  'installed.  To test extensions support, install '
                  'pyasn1 and retry')
    PYASN1_SUPPORT = False
    
from contrail.security.ca.impl import CertificateAuthority

if PYASN1_SUPPORT:
    from contrail.security.ca.subj_alt_name import GeneralNames
    
from contrail.security.ca.test import CertificateAuthorityBaseTestCase, THIS_DIR


class CertificateAuthorityTestCase(CertificateAuthorityBaseTestCase):
    """Test certificate authority class"""
    CERT_AUTHORITY_CLASS = CertificateAuthority
    CFG_FILEPATH = path.join(THIS_DIR, 'ca.cfg')
    
    def test01_issue_fqdn_cert_with_subj_alt_names(self):
        key_pair, cert_req, ca = self._create_ca_and_cert_req()
        
        not_before_nsecs = 0
        not_after_nsecs =  60*60*24*365*5
        
        cert = ca.issue_certificate(
                      cert_req, 
                      not_before_time_nsecs=not_before_nsecs, 
                      not_after_time_nsecs=not_after_nsecs, 
                      subject_alt_name='DNS:localhost, DNS:localhost.domain')
    
        
        s_key = crypto.dump_privatekey(crypto.FILETYPE_PEM, key_pair)
        open(path.join(THIS_DIR, 'my.key'), 'w').write(s_key)
        s_cert = crypto.dump_certificate(crypto.FILETYPE_PEM, cert)
        open(path.join(THIS_DIR, 'my.crt'), 'w').write(s_cert)

    if PYASN1_SUPPORT:
        def test02_check_ext(self):
                
            # Check for subject alternative names
            cert = crypto.load_certificate(
                                    crypto.FILETYPE_PEM, 
                                    open(path.join(THIS_DIR, 'my.crt')).read())
            
            for i in range(cert.get_extension_count()):
                ext = cert.get_extension(i)
                ext_name = ext.get_short_name()
                if ext_name == 'subjectAltName':
                    ext_dat = ext.get_data()
                    print ext_dat
                    dec = decode(ext_dat, asn1Spec=GeneralNames())
                    print dec
                    print dec[0].prettyPrint()
                    for i in range(len(dec[0])):
                        dns_name = str(
                                dec[0].getComponentByPosition(i).getComponent())
                        print dns_name

    def test03_create_from_keywords(self):
        ca = CertificateAuthority.from_keywords(
                            cert_filepath=self.__class__.CA_CERT_FILEPATH, 
                            key_filepath=self.__class__.CA_KEY_FILEPATH, 
                            key_passwd=self.__class__.CA_KEY_FILE_PASSWD,
                            serial_num_counter=11)
        self.assert_(ca, 'null ca object')
        self.assertEqual(ca.serial_num_counter, 11, 
                         'Error setting serial_num_counter')
        
    def test04_create_from_files(self):
        ca = CertificateAuthority.from_files(self.__class__.CA_CERT_FILEPATH, 
                            self.__class__.CA_KEY_FILEPATH, 
                            key_file_passwd=self.__class__.CA_KEY_FILE_PASSWD)
        self.assert_(ca, 'null ca object')
        self.assertIsInstance(ca.cert, crypto.X509, 
                              'ca.cert is not an X509 instance')
        self.assertIsInstance(ca.key, crypto.PKey, 
                              'ca.key is not an PKey instance')
        
    def test05_create_from_config(self):
        ca = CertificateAuthority.from_config(self.__class__.CFG_FILEPATH)
        self.assert_(ca, 'null ca object')
        self.assertIsInstance(ca.cert, crypto.X509, 
                              'ca.cert is not an X509 instance')
        self.assertIsInstance(ca.key, crypto.PKey, 
                              'ca.key is not an PKey instance')
        
    def test06_issue_cert_with_custom_ext(self):
        key_pair, cert_req, ca = self.__class__._create_ca_and_cert_req()
        
        not_before_nsecs = 0
        not_after_nsecs =  60*60*24*365*5

        cert = ca.issue_certificate(
                      cert_req, 
                      not_before_time_nsecs=not_before_nsecs, 
                      not_after_time_nsecs=not_after_nsecs, 
                      extensions=[('nsComment', 'my_cust_val', False)])

        s_key = crypto.dump_privatekey(crypto.FILETYPE_PEM, key_pair)
        open(path.join(THIS_DIR, 'my1.key'), 'w').write(s_key)
        s_cert = crypto.dump_certificate(crypto.FILETYPE_PEM, cert)
        open(path.join(THIS_DIR, 'my1.crt'), 'w').write(s_cert)
        
        
if __name__ == "__main__":
    unittest.main()


#/usr/bin/env python
'''
Created on Aug 10, 2012

@author: philipkershaw
'''
from OpenSSL import crypto


def gen_ca_cert(dn):
    ca_key = crypto.PKey()
    ca_key.generate_key(crypto.TYPE_RSA, 2048)

    ca_cert = crypto.X509()
    ca_cert.set_version(3)
    ca_cert.set_serial_number(1)
    subj = ca_cert.get_subject()
    for k, v in dn.items():
        setattr(subj, k, v)
        
    ca_cert.gmtime_adj_notBefore(0)       
    ca_cert.gmtime_adj_notAfter(24 * 60 * 60)
    ca_cert.set_issuer(ca_cert.get_subject())
    ca_cert.set_pubkey(ca_key)
    ca_cert.add_extensions([
      crypto.X509Extension("basicConstraints", True,
                                   "CA:true, pathlen:0"),
      crypto.X509Extension("keyUsage", True,
                                   "keyCertSign, cRLSign"),
      crypto.X509Extension("subjectKeyIdentifier", False, "hash",
                                   subject=ca_cert),
      ])
    ca_cert.sign(ca_key, "sha1")
    
    return ca_key, ca_cert

        
if __name__ == "__main__":
    from os import path
    import sys
    this_dir = path.abspath(path.dirname(__file__))
    
    if len(sys.argv) == 3:
        cakey_filepath, cacert_filepath = sys.argv[1:2]
    else:
        cakey_filepath = path.join(this_dir, 'myca.key')
        cacert_filepath = path.join(this_dir, 'myca.crt')
    
    dn = {'CN': 'NDG Test CA', 'O': 'NDG', 'OU': 'Security'}
    key_pair, cert = gen_ca_cert(dn)
    
    s_key = crypto.dump_privatekey(crypto.FILETYPE_PEM, key_pair)
    open(cakey_filepath, 'w').write(s_key)
    s_cert = crypto.dump_certificate(crypto.FILETYPE_PEM, cert)
    open(cacert_filepath, 'w').write(s_cert)




#!/usr/bin/env python
"""Distribution Utilities setup program for Certificate Authority Package

Contrail Project
"""
__author__ = "P J Kershaw"
__date__ = "24/10/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = """BSD - See LICENSE file in top-level directory"""
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = '$Id: setup.py 8244 2012-10-29 15:51:12Z pjkersha $'

# Bootstrap setuptools if necessary.
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


setup(
    name =            	'ContrailCA',
    version =         	'0.1.0',
    description =     	'Certificate Authority',
    long_description = 	'''\
Provides a simple implementation of a Certificate Authority.  It uses the 
PyOpenSSL for bindings to OpenSSL but also includes the ability to callout 
direct to an openssl command for more fine-grained control over the certificate 
issuing process if required.

The code has been developed for the Contrail Project, http://contrail-project.eu/

Prerequisites
=============
This has been developed and tested for Python 2.7.

Installation
============
Installation can be performed using easy_install or pip.

Configuration
=============
Examples are contained in ``contrail.security.ca.test``.
''',
    author =          	'Philip Kershaw',
    author_email =    	'Philip.Kershaw@stfc.ac.uk',
    maintainer =        'Philip Kershaw',
    maintainer_email =  'Philip.Kershaw@stfc.ac.uk',
#    url =             	'',
    platforms =         ['POSIX', 'Linux', 'Windows'],
    install_requires =  ['ndg_httpsclient'],
    
    # Required for Subject Alt Names unit test only
    extras_require = {'subjectAltName_support': 'pyasn1'},
    license =           __license__,
    test_suite =        'contrail.security.onlineca.client.test',
    packages =          find_packages(),
    package_data =      {
        'contrail.security.ca.test': [
            'README', '*.cfg', '*.crt', '*.key',
            'ca_config/serial', 'ca_config/index.txt', 
            'ca_config/newcerts/README', 'ca_config/*.crt', 'ca_config/*.key', 
            'ca_config/test-ca.cfg'
            ]
    },
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Topic :: Security',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Systems Administration :: Authentication/Directory',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    zip_safe = False
)



"""Module for Certificate Authority default implementation unit tests
"""
__author__ = "P J Kershaw"
__date__ = "20/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: $"
import unittest
from os import path
from os import urandom
import base64

from contrail.security.ca.cert_req import CertReqUtils

THIS_DIR = path.abspath(path.dirname(__file__))


class CertificateAuthorityBaseTestCase(unittest.TestCase):
    """Certificate authority unit test base class"""
    CA_CERT_FILEPATH = path.join(THIS_DIR, 'ca_config', 'myca.crt')
    CA_KEY_FILEPATH = path.join(THIS_DIR, 'ca_config', 'myca.key')
    CA_KEY_FILE_PASSWD = 'ndgtestca'
    CERT_AUTHORITY_CLASS = None
    MIN_KEY_NBITS = 1024
    
    @classmethod
    def create_cert_req(cls, dn=None):
        key_pair = CertReqUtils.create_key_pair()
        
        if dn is None:
            dn = {'CN': 'localhost', 'O': 'NDG', 'OU': 'Security'}
            
        cert_req = CertReqUtils.create_cert_req(dn, key_pair)
        
        return key_pair, cert_req
    
    @classmethod
    def _create_ca_and_cert_req(cls):
        key_pair, cert_req = cls.create_cert_req()
        ca = cls.CERT_AUTHORITY_CLASS.from_files(cls.CA_CERT_FILEPATH, 
                                                 cls.CA_KEY_FILEPATH, 
                                                 cls.CA_KEY_FILE_PASSWD)
        ca.min_key_nbits = cls.MIN_KEY_NBITS
        
        return key_pair, cert_req, ca



"""Certificate Authority default implementation
"""
__author__ = "P J Kershaw"
__date__ = "19/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: impl.py 8217 2012-10-24 16:26:04Z pjkersha $"
import logging
log = logging.getLogger(__name__)
    
from OpenSSL import crypto

from contrail.security.ca.cert_req import CertReqUtils
from contrail.security.ca.base import (AbstractCertificateAuthority, 
                                       CertificateAuthorityError)


class CertificateAuthorityCSRError(CertificateAuthorityError):
    """Error with input certificate signing request"""


class CertificateAuthority(AbstractCertificateAuthority):
    '''Certificate Authority implementation entirely based on PyOpenSSL bindings
    '''
    DEFAULT_NOT_BEFORE_TIME = 0
    DEFAULT_NOT_AFTER_TIME = 60*60*24*365*3 # 3 years
    
    __slots__ = (
        '__not_before_time_nsecs',
        '__not_after_time_nsecs',
        '__digest',
        '__certificate_version',
        '__ca_true',
        '__subject_alt_name'
    )
    def __init__(self):
        super(CertificateAuthority, self).__init__()
        
        self.__not_before_time_nsecs = self.__class__.DEFAULT_NOT_BEFORE_TIME 
        self.__not_after_time_nsecs = self.__class__.DEFAULT_NOT_AFTER_TIME 
        self.__digest = CertReqUtils.MESSAGE_DIGEST_TYPE_DEFAULT
        self.__certificate_version = self.__class__.CERTIFICATE_VERSION3
        self.__ca_true = False
        self.__subject_alt_name = False
        
    @property
    def not_before_time_nsecs(self):
        """Default not before time in seconds for certs issued"""
        return self.__not_before_time_nsecs

    @not_before_time_nsecs.setter
    def not_before_time_nsecs(self, value):
        if not isinstance(value, (long, int)):
            raise TypeError('Expecting int or long type for '
                            '"not_before_time_nsecs" got %r type' % type(value))
        self.__not_before_time_nsecs = long(value)
        
    @property
    def not_after_time_nsecs(self):
        """Default not after time in seconds for certs issued"""
        return self.__not_after_time_nsecs

    @not_after_time_nsecs.setter
    def not_after_time_nsecs(self, value):
        if not isinstance(value, (long, int)):
            raise TypeError('Expecting int or long type for '
                            '"not_after_time_nsecs" got %r type' % type(value))
        self.__not_after_time_nsecs = long(value)
  
    @property
    def digest(self):
        return self.__digest
    
    @digest.setter
    def digest(self, value):
        if not isinstance(value, basestring):
            raise TypeError('Expecting string type for "digest" '
                            'got %r type' % type(value))
        self.__digest = value
  
    @property
    def certificate_version(self):
        return self.__certificate_version
    
    @certificate_version.setter
    def certificate_version(self, value):
        if not isinstance(value, basestring):
            raise TypeError('Expecting string type for "certificate_version" '
                            'got %r type' % type(value))
        self.__certificate_version = value
                        
    @property
    def ca_true(self):
        """Set to true to issue *CA* certificates by default"""
        return self.__ca_true

    @ca_true.setter
    def ca_true(self, value):
        if isinstance(value, basestring):
            self.__ca_true = value.lower() in ('1', 'true')
            
        elif isinstance(value, (long, int)):
            self.__ca_true = long(value)
        else:
            raise TypeError('Expecting int or long type for '
                            '"ca_true" got %r type' % type(value))
        
    @property
    def subject_alt_name(self):
        """Set to true to set subject alt name in certificates by default"""
        return self.__subject_alt_name

    @subject_alt_name.setter
    def subject_alt_name(self, value):
        if isinstance(value, basestring):
            self.__subject_alt_name = value.lower() in ('1', 'true')
            
        elif isinstance(value, (long, int)):
            self.__subject_alt_name = long(value)
        else:
            raise TypeError('Expecting int or long type for '
                            '"ca_true" got %r type' % type(value))        
                
    def issue_certificate(
        self, 
        cert_req,
        subject_name=None, 
        not_before_time_nsecs=DEFAULT_NOT_BEFORE_TIME, 
        not_after_time_nsecs=DEFAULT_NOT_AFTER_TIME, 
        digest=CertReqUtils.MESSAGE_DIGEST_TYPE_DEFAULT,
        certificate_version=AbstractCertificateAuthority.CERTIFICATE_VERSION3,
        ca_true=False,
        subject_alt_name=False,
        extensions=None):
        """
        Generate a certificate given a certificate request.
    
        @param cert_req: Certificate request to use
        @param not_before_time_nsecs: Timestamp (relative to now) when the 
        certificate starts being valid
        @param subject_name: set alternate subject name to one specified in the
        certificate request
        @type subject_name: OpenSSL.crypto.X509Name
        @type not_before_time_nsecs: int or long
        @param not_after_time_nsecs: Timestamp (relative to now) when the certificate
        stops being valid
        @type not_after_time_nsecs: int or long
        @param digest: Digest method to use for signing, default is md5
        @param ca_true: set to True to set CA:true in the basic constraints 
        extension
        @type ca_true: bool
        @return: The signed certificate in an X.509 object
        """
        
        # Check number of bits in key
        pkey = cert_req.get_pubkey()
        pkey_nbits = pkey.bits()
        if pkey_nbits < self.min_key_nbits:
            raise CertificateAuthorityCSRError('Certificate signing request '
                                               'must use a key with at least '
                                               '%d bits, input request has a '
                                               'key with %d bits' % 
                                               (self.min_key_nbits, pkey_nbits))
        
        cert = crypto.X509()
        
        # Update serial number from value stored in file if available
        if self.serial_filepath:
            self._read_serial_file()
        
        cert.set_serial_number(self.serial_num_counter)
        
        cert.gmtime_adj_notBefore(not_before_time_nsecs)
        cert.gmtime_adj_notAfter(not_after_time_nsecs)

        cert.set_issuer(self.cert.get_subject())
        
        if subject_name is None:
            subject_name = cert_req.get_subject()
            
        cert.set_subject(subject_name)
        cert.set_pubkey(cert_req.get_pubkey())
        
        cert.set_version(certificate_version)
        
        # Certificate extensions
        if ca_true:
            basic_constraints = 'CA:true'
        else:
            basic_constraints = 'CA:false'
            
        # Add basic constraints as first element of extensions list
        basic_constraints_ext = crypto.X509Extension('basicConstraints', 
                                                     True, 
                                                     basic_constraints)
        x509_extensions = [basic_constraints_ext]
            
        # Check for a subject alt names extension, if present add as is.
        if isinstance(subject_alt_name, basestring):
            subject_alt_name_ext = crypto.X509Extension('subjectAltName', 
                                                        False, 
                                                        subject_alt_name)
            x509_extensions.append(subject_alt_name_ext)
            
        if extensions:
            x509_extensions += self._add_certificate_ext(cert, extensions)
            
        cert.add_extensions(x509_extensions)
        
        cert.sign(self.key, digest)
        
        # Serial number is a counter
        self.serial_num_counter += 1
        
        if self.serial_filepath is not None:
            self._write_serial_file()
            
        if log.isEnabledFor(logging.INFO):
            dn = ''.join(["/%s=%s" % (k, v) 
                          for k,v in cert.get_subject().get_components()])
            
            log.info('Issuing certificate with subject %s', dn)
        
        return cert 
    
    def _add_certificate_ext(self, cert, extensions):
        """Add certificate extension - derived classes can override to customise
        behaviour
        """
        x509_extensions = []
        for ext_name, ext_val, ext_crit in extensions:
            x509_cust_ext = crypto.X509Extension(ext_name, 
                                                 ext_crit, 
                                                 str(ext_val))
            x509_extensions.append(x509_cust_ext)
            
        return x509_extensions



"""Certificate Authority package certificate request helper module
"""
__author__ = "P J Kershaw"
__date__ = "19/09/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: cert_req.py 8151 2012-09-20 20:58:44Z pjkersha $"
from OpenSSL import crypto


class CertReqUtils(object):
    """Utility class containing helper functions for use with Certificate
    Authority"""
    PRIKEY_NBITS_DEFAULT = 1024
    MESSAGE_DIGEST_TYPE_DEFAULT = "md5"
        
    @staticmethod
    def create_key_pair(nbits_for_key=PRIKEY_NBITS_DEFAULT):
        """Generate key pair and return as PEM encoded string
        @type nbits_for_key: int
        @param nbits_for_key: number of bits for private key generation - 
        default is 2048
        @rtype: OpenSSL.crypto.PKey
        @return: public/private key pair
        """
        key_pair = crypto.PKey()
        key_pair.generate_key(crypto.TYPE_RSA, nbits_for_key)
        
        return key_pair
            
    @staticmethod
    def create_cert_req(dn, 
                        key_pair, 
                        message_digest=MESSAGE_DIGEST_TYPE_DEFAULT):
        """Create a certificate request.
        
        @param dn: The distinguished name of the subject of the request, 
        possible arguments are:
          C     - Country name
          SP    - State or province name
          L     - Locality name
          O     - Organization name
          OU    - Organizational unit name
          CN    - Common name
          email - E-mail address
        @type dn: dict
        @type key_pair: string/None
        @param key_pair: public/private key pair
        @type message_digest: basestring
        @param message_digest: message digest type - default is MD5
        @return certificate request PEM text and private key PEM text
        @rtype: base string
        """
        
        # Check all required certificate request DN parameters are set                
        # Create certificate request
        cert_req = crypto.X509Req()
        subj = cert_req.get_subject()
    
        for k, v in dn.items():
            setattr(subj, k, v)
        
        # Create public key object
        cert_req.set_pubkey(key_pair)
        
        # Add the public key to the request
        cert_req.sign(key_pair, message_digest)
        
        return cert_req
